@@ -1,6 +1,7 @@
 import asyncio
 import dataclasses
 import queue
+from random import randint
 from typing import Any
 from typing import Dict
 from typing import List
@@ -21,8 +22,9 @@ class Event:
 
 
 async def get(hub):
-    bus = await hub.evbus.broker.init()
-    return await bus.get()
+    bus: asyncio.PriorityQueue = await hub.evbus.broker.init()
+    queue_item = await bus.get()
+    return queue_item[1]
 
 
 async def init(hub):
@@ -31,10 +33,11 @@ async def init(hub):
     """
     # Once we are running in the context of asyncio, replace the synchronous queue with an async queue
     if isinstance(hub.evbus.BUS, queue.Queue):
-        new_bus = asyncio.Queue()
+        new_bus = asyncio.PriorityQueue()
         while not hub.evbus.BUS.empty():
-            event = hub.evbus.BUS.get()
-            new_bus.put_nowait(event)
+            queue_item = hub.evbus.BUS.get()
+            # The item should already be a tuple here
+            new_bus.put_nowait(queue_item)
 
         hub.evbus.BUS = new_bus
     return hub.evbus.BUS
@@ -48,17 +51,16 @@ async def put(hub, body: str, profile: str = "default"):
     :param profile: The ctx profile name that should be used for this message
     :param serial_plugin: The pop-serial plugin used to serialize data
     """
-    serialized: bytes = hub.serial[hub.evbus.SERIAL_PLUGIN].dump(body)
-    bus: asyncio.Queue = await hub.evbus.broker.init()
-    await bus.put(Event(body=serialized, profile=profile))
+    bus: asyncio.PriorityQueue = await hub.evbus.broker.init()
+    await bus.put(_create_queue_item(hub, body, profile))
 
 
 def put_nowait(hub, body: str, profile: str = "default"):
     """
     Put an event on the bus as a single data class
     """
-    serialized: bytes = hub.serial[hub.evbus.SERIAL_PLUGIN].dump(body)
-    hub.evbus.BUS.put_nowait(Event(body=serialized, profile=profile))
+    bus: asyncio.PriorityQueue = hub.evbus.BUS
+    bus.put_nowait(_create_queue_item(hub, body, profile))
 
 
 async def propagate(
@@ -88,12 +90,44 @@ async def propagate(
                     # Treat the profile name as a pattern, and see if the event profile name matches
                     if hub.match[match_plugin].find(name=event.profile, pattern=name):
                         ctx = data.NamespaceDict(acct=profile[name])
-                        coros.append(plugin.publish(ctx=ctx, body=event.body))
+                        # co-routines cannot be awaited more than once, but tasks can be
+                        # We need plugin and event metadata so that we can send the failure to error callback handler
+                        publish_task = asyncio.create_task(
+                            hub.evbus.broker.wrap(
+                                getattr(plugin, "__name__", None),
+                                event,
+                                contexts,
+                                coro=plugin.publish(ctx=ctx, body=event.body),
+                            )
+                        )
+                        coros.append(publish_task)
 
-    for task in asyncio.as_completed(coros):
-        # Don't stop the listener for a failure
-        try:
-            await task
-        except Exception as e:
-            # Log at the lowest level in case log messages are being sent to a queue
-            hub.log.trace(f"Event failed to propagate: {e.__class__.__name__}: {e}")
+        await asyncio.gather(*coros)
+
+
+async def wrap(hub, plugin_name, event, contexts, coro):
+    try:
+        return await coro
+    except Exception as e:
+        # Log all failures, in case log messages are being sent to a queue the error message will be formatted
+        hub.log.error(
+            f"Event failed to propagate on plugin {plugin_name} with profile {event.profile}: {e.__class__.__name__}: {e}"
+        )
+        if hub.OPT.evbus.error_callback_ref:
+            hub.log.trace(
+                f"The error from {plugin_name} is being sent to error callback handler {hub.OPT.evbus.error_callback_ref}"
+            )
+            error_data = {
+                "exception": e,
+                "plugin": plugin_name,
+                "profile": event.profile,
+                "body": event.body,
+                "context": contexts,
+            }
+            await hub.pop.loop.wrap(hub[hub.OPT.evbus.error_callback_ref], error_data)
+
+
+def _create_queue_item(hub, body: str, profile: str = "default") -> tuple:
+    serialized: bytes = hub.serial[hub.evbus.SERIAL_PLUGIN].dump(body)
+    # Duplicate priority is okay as the order is not important when publishing event
+    return randint(0, 100), Event(body=serialized, profile=profile)
